// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System.IO;
using System.Drawing;
using System.Text.RegularExpressions;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Annotations;
using GrapeCity.Documents.Pdf.TextMap;
using GrapeCity.Documents.Pdf.AcroForms;

namespace DsPdfWeb.Demos
{
    // このサンプルでは、テキストマップを使用して PDF 内の特定のコンテンツを
    // 検索し、それを墨消しの対象としてマークする方法を示しています。
    // なお、 ApplyRedact サンプルでは、このサンプルで追加された墨消し注釈に
    // 実際にデータを消去するために墨消しを適用する方法を確認できます。
    // このサンプルで使用される PDF は TimeSheet によって作成されたものです。
    public class FindAndRedact
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            using (var fs = File.OpenRead(Path.Combine("Resources", "PDFs", "TimeSheet.pdf")))
            {
                doc.Load(fs);

                // メモ：Acrobat では、デジタル署名された文書に編集を適用することは許可されて
                // いないため、まず既存の署名を見つけて削除します。
                removeSignatureFields(doc.AcroForm.Fields);

                // ページをループして、短い日付のように見えるものをすべて削除します。
                foreach (var page in doc.Pages)
                {
                    var tmap = page.GetTextMap();
                    foreach (ITextLine tline in tmap)
                    {
                        if (Regex.Match(tline.Text.Trim(), @"\d+[/-]\w+[/-]\d").Success)
                        {
                            var redact = new RedactAnnotation()
                            {
                                Rect = tline.GetCoords().ToRect(),
                                MarkBorderColor = Color.Red,
                                MarkFillColor = Color.Yellow,
                                Page = page
                            };
                            // まだ redact.Page = page を設定していない場合は、次のようにします:
                            // page.Annotations.Add(redact);
                        }
                    }
                }
                // PDF ドキュメントを保存します。
                doc.Save(stream);
                return doc.Pages.Count;

                // このコードは、RemoveSignatureFields サンプルからのものです。
                void removeSignatureFields(FieldCollection fields)
                {
                    for (int i = fields.Count - 1; i >= 0; --i)
                    {
                        removeSignatureFields(fields[i].Children);
                        if (fields[i] is SignatureField)
                            fields.RemoveAt(i);
                    }
                }
            }
        }

        // デモのためのビューワのオプションです。
        public static GcPdfViewerSupportApiDemo.Models.PdfViewerOptions PdfViewerOptions
        {
            get => new GcPdfViewerSupportApiDemo.Models.PdfViewerOptions(
                GcPdfViewerSupportApiDemo.Models.PdfViewerOptions.Options.AnnotationEditorPanel |
                GcPdfViewerSupportApiDemo.Models.PdfViewerOptions.Options.ActivateAnnotationEditor,
                viewerTools: new string[] { "save", "edit-redact-apply", "$split", "text-selection", "pan", "$zoom", "$fullscreen", "print", "rotate", "download", "doc-properties", "about" },
                annotationEditorTools: new string[] { "save", "edit-redact", "edit-redact-apply" });
        }
    }
}
